#include <cstdio>
#include <vector>
#include <complex>
#include <cmath>
#include <algorithm>

using namespace std;
typedef complex<double> complx;

struct Point {
    double freq;
    complx imp;
};

struct Data : public vector<Point> {
    void load_file(FILE* f) {
        Point p;
        double r, i;
        while( fscanf(f, "%lf %lf %lf", &p.freq, &r, &i) == 3 ) {
            p.imp = complx(r, i);
            push_back(p);
        }
    }
};

enum SpecType {LOWER, HIGHER};

struct Spec {
    double   freq;
    double   value;
    SpecType type;

    bool operator<(const Spec& o) const {
        if(freq  != o.freq ) return freq  < o.freq;
        if(value != o.value) return value < o.value;
        return type < o.type;
    }
};

/* start: start value ( 10^start), inclusive. end = end value (10^end), inclusive. points: points per decade */
static vector<double> logspace(unsigned start, unsigned end, unsigned points) {
    vector<double> freqs;
    double factor = 10.0/points;

    for(unsigned e = start; e < end; ++e)
        for(unsigned p = 1; p <= points; ++p) 
            freqs.push_back(p * factor * pow(10, e));

    return freqs;
}

struct Constraints : public vector<Spec> {
    void load_file(FILE* f) {
        unsigned start, end, points;
        double value;
        char spec[2];

        while( fscanf(f, "%u %u %u %lf %s", &start, &end, &points, &value, spec) == 5 )
            fill_logspace(start, end, points, value, spec[0] == 'l' ? LOWER : HIGHER);

        sort( begin(), this->end() );
    }


    void fill_logspace(unsigned start, unsigned end, unsigned points, double value, SpecType type) {
        vector<double> freqs = logspace(start, end, points);

        for(unsigned i = 0, e = freqs.size(); i < e; ++i) {
            Spec s;
            s.freq  = freqs[i];
            s.value = value;
            s.type  = type;
            push_back(s);
        }
    }
};
