#include <complex>
#include <cstdio>

using namespace std;

class Howland {
public:
    Howland(const OpAmp& op, double R, double r){ // R,r em kOhms
        R *= 1e3;
        r *= 1e3;
        this->op = op;
        this->R  = R;
        this->r  = r;

        G[0] = 1 / op.Ro;
        G[1] = 1.0 / (R + r);
        G[2] = G[5] = G[3] = 1.0 / R;
        G[4] = 1.0 / r;
    }

    complx Zo(double freq) const {
        double omega = 2*M_PI*freq;

        complx Yi(1 / op.Ri, (omega * op.Ci));
        complx A = op.Ao / complx(1, omega/op.Wc);

        return 2.0 * ( ((G[3]+G[1])*G[5]*G[0]*A+((G[5]+G[3]+G[2]+G[1])*Yi+(G[3]+G[1])*G[5]+G[2]*G[3]+G[1]*G[2])*G[0]+
        ((G[4]+G[3]+G[2]+G[1])*G[5]+(G[3]+G[2]+G[1])*G[4])*Yi+((G[3]+G[1])*G[4]+G[2]*G[3]+G[1]*G[2])*G[5]+(G[2]*G[3]+G[1]*G[2])*G[4])/(
        ((G[1]*G[4]+G[1]*G[3])*G[5]-G[2]*G[3]*G[4])*G[0]*A+(((G[4]+G[3])*G[5]+(G[3]+G[2]+G[1])*G[4]+(G[2]+G[1])*G[3])*Yi+((G[3]
        +G[1])*G[4]+G[1]*G[3])*G[5]+(G[2]*G[3]+G[1]*G[2])*G[4]+G[1]*G[2]*G[3])*G[0]
        +(((G[2]+G[1])*G[4]+(G[2]+G[1])*G[3])*G[5]+(G[2]+G[1])*G[3]*G[4])*Yi+
        (((G[2]+G[1])*G[3]+G[1]*G[2])*G[4]+G[1]*G[2]*G[3])*G[5]+G[1]*G[2]*G[3]*G[4]) );
    }

    complx Io(double freq, double Vi) const {
        double omega = 2*M_PI*freq;
        complx A  = op.Ao / complx(1, omega/op.Wc);
        complx Zi = op.Ri / (1.0 + complx(0, omega) * op.Ri * op.Ci);
        double Zo = op.Ro;

        complx num = (2.0*Zi*A-r-2*Zo)*R*R + (Zi*r*A-r*r-3.0*Zo*r-2.0*Zi*Zo)*R - Zo*r-Zi*Zo*r;
        complx den = 4.0*(r+Zo)*R*R*R + (Zi*(2.0*r*A+4.0*Zo) + 3.0*r*r + (6.0*Zo+4.0*Zi)*r)*R*R + (Zi*r*(r*A+4.0*Zo)+2.0*(Zo+Zi)*r*r)*R + Zi*Zo*r*r;

        return -(num / den) * Vi * 2.0;
    }

    complx Il(double freq, double Vi, double l) const {
        complx io = Io(freq, Vi);
        complx zo = Zo(freq);

        return (io*zo)/(zo+l);
    }

    void freq_sweep(const vector<double>& freqs) const {
        for(unsigned i = 0, e = freqs.size(); i < e; ++i) {
            double f = freqs[i];
            complx z  = Zo(f);
            printf("%12.1f %22.12f\n", f, abs(z));
        }
    }

    void freq_sweep(const Data& data) const {
        for(unsigned i = 0, e = data.size(); i < e; ++i) {
            double f = data[i].freq;
            complx z = Zo(f);
            printf("%12.1f %22.12f %22.12f\n", f, real(z), imag(z));
        }
    }

    void freq_sweep(const Constraints& cons) const {
        double last = -1;

        for(unsigned i = 0, e = cons.size(); i < e; ++i) {
            double f = cons[i].freq;
            if(f != last) {
                last = f;
                complx z = Zo(f);
                printf("%12.1f %22.12f %22.12f\n", f, real(z), imag(z));
            }
        }
    }

    double mse(const Data& data) const {
        double acc = 0;

        for(unsigned i = 0, e = data.size(); i < e; ++i) {
            double err = abs(data[i].imp - Zo(data[i].freq));
            acc += ( err / abs(data[i].imp) );
        }

        return acc / data.size();
    }

    double mse(const Constraints& cons) const {
        double acc = 0;

        for(unsigned i = 0, e = cons.size(); i < e; ++i) {
            double out = abs( Zo(cons[i].freq) );
            double err = 0;

            if(cons[i].type == LOWER)
                err = out - cons[i].value;
            else
                err = cons[i].value - out;

            if(err > 0) acc += err * err;
        }

        return acc / cons.size();
    }

    double mse_il(const Constraints& cons, double Vi, double l, bool detail = false) const {
        double acc = 0;

        for(unsigned i = 0, e = cons.size(); i < e; ++i) {
            double out = abs( Il(cons[i].freq, Vi, l) );
            double err = 0;

            if(cons[i].type == LOWER)
                err = out - cons[i].value;
            else
                err = cons[i].value - out;

            if(err > 0) {
                acc += err * err;
                
                if(detail)
                    fprintf(stderr, "error: %g mA, on freq: %g Hz\n", err, cons[i].freq);
                
            }
        }

        return acc / cons.size();
    }

    void freq_sweep_il(const vector<double>& freqs, double Vi, double l) const {
        for(unsigned i = 0, e = freqs.size(); i < e; ++i) {
            complx il = Il(freqs[i], Vi, l);
            printf("%12.1f %22.12f\n", freqs[i], abs(il));
        }
    }
    
    void freq_sweep_io(const vector<double>& freqs, double Vi) const {
        for(unsigned i = 0, e = freqs.size(); i < e; ++i) {
            complx io = Io(freqs[i], Vi);
            printf("%12.1f %22.12f\n", freqs[i], abs(io));
        }
    }

    double cost() {
        return op.cost();
    }

private:
    double G[6], R, r;
    OpAmp op;
};
